# http://glowingpython.blogspot.com/2011/05/four-ways-to-compute-google-pagerank.html
from numpy import *

def powerMethodBase(A,x0,iter):
 """ basic power method """
 for i in range(iter):
  x0 = dot(A,x0)
  x0 = x0/linalg.norm(x0,1)
 return x0

def powerMethodBase1(A,x0,thresh,iter):
  """ basic power method """
  x0 = x0/linalg.norm(x0,1)
  x1 = x0
  for i in range(iter):
   x0 = dot(A,x0)
   x0 = x0/linalg.norm(x0,1)
   stop = linalg.norm(x0-x1,1)
   if stop < thresh:
    print "Stop ",stop, " After ", i, " iterations"
    break
   x1 = x0
  return x0
 
def powerMethod(A,x0,m,iter):
 """ power method modified to compute
     the maximal real eigenvector 
     of the matrix M built on top of the input matrix A """
 n = A.shape[1]
 delta = m*(array([1]*n,dtype='float64')/n) # array([1]*n is [1 1 ... 1] n times
 for i in range(iter):
  x0 = dot((1-m),dot(A,x0)) + delta
 return x0

def maximalEigenvector(A):
 """ using the eig function to compute eigenvectors """
 n = A.shape[1]
 w,v = linalg.eig(A)
 return abs(real(v[:n,0])/linalg.norm(v[:n,0],1))

def linearEquations(A,m):
 """ solving linear equations 
     of the system (I-(1-m)*A)*x = m*s """
 n = A.shape[1]
 C = eye(n,n)-dot((1-m),A)
 b = m*(array([1]*n,dtype='float64')/n)
 return linalg.solve(C,b)

def getTeleMatrix(A,m):
 """ return the matrix M
     of the web described by A """
 n = A.shape[1]
 S = ones((n,n))/n
 return (1-m)*A+m*S

A = array([ [0,     0,     0,     1, 0, 1],
            [1/2.0, 0,     0,     0, 0, 0],
            [0,     1/2.0, 0,     0, 0, 0],
            [0,     1/2.0, 1/3.0, 0, 0, 0],
            [0,     0,     1/3.0, 0, 0, 0],
            [1/2.0, 0,     1/3.0, 0, 1, 0 ] ])

n = A.shape[1] # A is n x n
m = 0.15
M = getTeleMatrix(A,m)

x0 = ones(n)/ n
x1 = powerMethod(A,x0,m,130)
x2 = powerMethodBase(M,x0,130)
x5 = powerMethodBase1(M,x0,1.0e-6, 130)
x3 = maximalEigenvector(M)
if m > 0.001:
	x4 = linearEquations(A,m)

# comparison of the four methods
labels = range(1,6)
print "1 ", x1
print "2 ", x2
print "3 ", x3
if m > 0.001:
	print "4 ", x4
print "5 ", x5

H = array([ [0,     0, 0,     0,      0,     0, 1.0/3.0, 0],
            [1/2.0, 0, 1/2.0, 1.0/3.0,0,     0, 0,       0],
            [1/2.0, 0, 0,     0,      0,     0, 0,       0],
            [0,     1, 0,     0,      0,     0, 0,       0],
            [0,     0, 1/2.0, 1/3.0,  0,     0, 1/3.0,   0],
            [0,     0, 0,     1/3.0,  1/3.0, 0, 0,     1/2.0 ],
            [0,     0, 0,     0,      1/3.0, 0, 0,     1/2.0 ],
            [0,     0, 0,     0,      1/3.0, 1, 1/3.0, 0 ] ])
            
n = H.shape[1] # H is n x n
m = 0.0
M = getTeleMatrix(H,m)

x0 = ones(n)/ n
x1 = powerMethod(H,x0,m,130)
x2 = powerMethodBase(M,x0,130)
x5 = powerMethodBase1(M,x0,1.0e-6, 130)
x3 = maximalEigenvector(M)
if m > 0.001:
	x4 = linearEquations(H,m)

# comparison of the four methods
labels = range(1,6)
print "1 ", x1
print "2 ", x2
print "3 ", x3
if m > 0.001:
	print "4 ", x4
print "5 ", x5
